// $Ringlet: cs/misc/sharelist/ShareMain.cs,v 1.1.1.1 2003/10/20 07:18:36 roam Exp $

using System;
using System.Collections.Specialized;
using System.Management;

namespace sharelist
{
	/// <summary>
	/// The main program class: list the shares of a specified host.
	/// </summary>
	class ShareMain
	{
		/// <summary>
		/// Parse the command-line options.
		/// </summary>
		/// <param name="args">the command-line options</param>
		/// <param name="coll">the collection to store the data into</param>
		/// <returns>True on success, false if an error occurred.</returns>
		protected static bool ParseOpts(string[] args, ref NameValueCollection coll)
		{
			uint i;
			string var;

			for (i = 0; i < args.Length; i++)
			{
				var = null;
				switch (args[i])
				{
					case "-u":
						var = "username";
						break;

					case "-p":
						var = "password";
						break;

					case "-H":
						var = "hostname";
						break;

					case "-h":
						return (false);
						
					default:
						Console.Error.WriteLine("Unrecognized parameter {0}", args[i]);
						return (false);
				}
				if (var != null)
				{
					if (i == args.Length - 1)
					{
						Console.Error.WriteLine("Option {0} requires a parameter", args[i]);
						return (false);
					}
					coll[var] = args[i + 1];
					i++;
				}
			}
			return (true);
		}

		/// <summary>
		/// Display program usage information.
		/// </summary>
		protected static void Usage()
		{
			Console.Error.WriteLine(
@"Usage: sharelist [-h] [-u username] [-p password] [-H host]");
		}

		/// <summary>
		/// List the shares of a specified host (or the local machine)
		/// </summary>
		/// <param name="coll">the options collection: username,
		/// password, hostname to query</param>
		protected static void ListShares(NameValueCollection coll)
		{
			ConnectionOptions opts;
			ManagementScope scope;
			ManagementObjectSearcher mgmt;
			string path;
			ObjectQuery q;
			
			mgmt = null;
			try
			{
				opts = new ConnectionOptions();
				if (coll["username"] != null)
					opts.Username = coll["username"];
				if (coll["password"] != null)
					opts.Password = coll["password"];
				if (coll["hostname"] != null)
					path = "\\\\" + coll["hostname"];
				else
					path = "";
				scope = new ManagementScope(path + "\\root\\cimv2", opts);
				scope.Connect();
				q = new ObjectQuery("select * from win32_share");
				mgmt = new ManagementObjectSearcher(scope, q);
				foreach (object o in mgmt.Get())
				{
					ManagementObject share;

					share = o as ManagementObject;
					if (share != null)
						Console.WriteLine(
						    "Found share: {0}",
						    share["Name"]);
					else
					{
						Console.WriteLine(
						    "Found object, type {0}:",
						    o.GetType().FullName);
						Console.WriteLine("{0}",
						    o.ToString());
					}
				}
				Console.WriteLine("End of share list");
			}
			catch (Exception ex)
			{
				Console.Error.WriteLine(
				    "Exception:\n{0}\n{1}",
				    ex.Message, ex.StackTrace);
			}
		}

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			NameValueCollection coll;

			coll = new NameValueCollection();
			if (!ParseOpts(args, ref coll))
			{
				Usage();
				return;
			}
			ListShares(coll);
		}
	}
}
